package cn.yyxx.columbus.core.impl

import android.app.Activity
import android.app.Application
import android.content.Context
import android.content.Intent
import android.text.TextUtils
import cn.yyxx.columbus.base.entity.SdkChargeInfo
import cn.yyxx.columbus.base.entity.SdkRoleInfo
import cn.yyxx.columbus.base.internal.ICallback
import cn.yyxx.columbus.base.internal.IInitialize
import cn.yyxx.columbus.base.utils.Logger
import cn.yyxx.columbus.base.utils.ParamsUtils
import cn.yyxx.columbus.core.entity.SdkBackLoginInfo
import cn.yyxx.columbus.core.entity.bean.init.InitBean
import cn.yyxx.columbus.core.impl.login.LoginActivity
import cn.yyxx.columbus.core.internal.IImplCallback
import cn.yyxx.columbus.core.network.Host
import cn.yyxx.support.DensityUtils
import cn.yyxx.support.device.DeviceInfoUtils
import cn.yyxx.support.gaid.GAIDUtils
import cn.yyxx.support.hawkeye.LogUtils
import cn.yyxx.support.hawkeye.OwnDebugUtils
import cn.yyxx.support.hawkeye.ToastUtils
import java.util.concurrent.atomic.AtomicInteger

/**
 * @author #Suyghur.
 * Created on 2021/06/16
 */
class SdkBridgeImpl(context: Context) {


    private var hasReadGaid = false

    @Volatile
    private var timeCount = AtomicInteger(0)

    init {
        Host.initHostModel(context)
        LogUtils.DEBUG = OwnDebugUtils.isOwnDebug(context, "yyxx_cfg.properties", "yyxx_game", "YYXX_OWN_DEBUG")
    }

    fun attachBaseContext(application: Application, context: Context) {
        Logger.initZap(application)
        Logger.i("Columbus attachBaseContext ...")
        GAIDUtils.initGoogleAdid(application) { code, _ ->
            if (code == 0) {
                Logger.i("谷歌框架可以访问，请求gaid")
                SdkDrive.getInstance().setParam("device_id", GAIDUtils.getGoogleAdid())
            } else {
                Logger.e("谷歌框架不可访问，使用android id替代")
                SdkDrive.getInstance().setParam("device_id", DeviceInfoUtils.getAndroidDeviceId(application))
            }
            hasReadGaid = true
        }
    }

    fun initApplication(application: Application) {
        Logger.i("Columbus initApplication ...")
        SdkDrive.getInstance().initSdkDrive(application)
    }

    fun initialize(activity: Activity, isLandscape: Boolean, callback: ICallback, initCallback: IInitialize) {
        Logger.i("Columbus initialize ...")
        isLand = isLandscape
        if (TextUtils.isEmpty(ParamsUtils.getGcpCode(activity))) {
            Logger.e("初始化失败，参数异常，请检查yyxx_cfg.properties中YYXX_GCP_CODE的值")
            initCallback.onResult(-1, "初始化失败，参数异常，请检查yyxx_cfg.properties中YYXX_GAME_CODE的值")
            callback.onResult(-1, "初始化失败，参数异常，请检查yyxx_cfg.properties中YYXX_GCP_CODE的值")
            return
        }

        if (TextUtils.isEmpty(ParamsUtils.getGameCode(activity))) {
            Logger.e("初始化失败，参数异常，请检查yyxx_cfg.properties中YYXX_GAME_CODE的值")
            initCallback.onResult(-1, "初始化失败，参数异常，请检查yyxx_cfg.properties中YYXX_GAME_CODE的值")
            callback.onResult(-1, "初始化失败，参数异常，请检查yyxx_cfg.properties中YYXX_GAME_CODE的值")
            return
        }

        if (TextUtils.isEmpty(ParamsUtils.getGoogleClientId(activity))) {
            Logger.e("初始化失败，参数异常，请检查yyxx_cfg.properties中YYXX_SERVER_CLIENT_ID的值")
            initCallback.onResult(-1, "初始化失败，参数异常，请检查yyxx_cfg.properties中YYXX_GAME_CODE的值")
            callback.onResult(-1, "初始化失败，参数异常，请检查yyxx_cfg.properties中YYXX_SERVER_CLIENT_ID的值")
            return
        }

        //获取当前屏幕尺寸
        SdkDrive.getInstance().setParam("screen", DensityUtils.getResolutionByFullScreen(activity))

        if (!hasReadGaid) {
            Logger.e("还未完成gaid加载，将延迟初始化")
            //TODO showInitDialog
            Thread {
                while (!hasReadGaid) {
                    try {
                        Logger.e("还未完成gaid加载，延迟1s初始化，$hasReadGaid")
                        Thread.sleep(1000)
                        timeCount.getAndIncrement()
                    } catch (e: InterruptedException) {
                        e.printStackTrace()
                    }
                }
                activity.runOnUiThread {
                    if (TextUtils.isEmpty(GAIDUtils.getGoogleAdid())) {
                        Logger.e("加载gaid失败，共延迟${timeCount}s初始化，将进行初始化操作...")
                    } else {
                        Logger.i("加载gaid成功，共延迟${timeCount}s初始化，将进行初始化操作...")
                    }
                    //TODO hideInitDialog
                    startSdkInit(activity, callback, initCallback)
                }
            }
        } else {
            Logger.i("加载gaid成功，开始初始化...")
            startSdkInit(activity, callback, initCallback)
        }
    }

    private fun startSdkInit(activity: Activity, callback: ICallback, initCallback: IInitialize) {
        val test =
            "{\"notice_cfg\":{\"switch\":1,\"url\":\"https://www.baidu.com\",\"show_count\":1},\"privacy_cfg\":{\"switch\":1,\"url\":\"https://www.baidu.com\"},\"float_cfg\":{\"switch\":1,\"float_icon_url\":\"https://www.baidu.com\",\"feature_cfg\":{\"member_cfg\":{\"switch\":1,\"icon_url\":\"https://www.baidu.com\",\"url\":\"https://www.baidu.com\"},\"gif_cfg\":{\"switch\":1,\"icon_url\":\"https://www.baidu.com\",\"url\":\"https://www.baidu.com\"},\"gm_cfg\":{\"switch\":1,\"icon_url\":\"https://www.baidu.com\",\"url\":\"https://www.baidu.com\"},\"charge_cfg\":{\"switch\":1,\"icon_url\":\"https://www.baidu.com\",\"url\":\"https://www.baidu.com\"},\"invite_cfg\":{\"switch\":1,\"icon_url\":\"https://www.baidu.com\",\"url\":\"https://www.baidu.com\"}}},\"reward_id\":\"com.yyxx.yzcjl60\",\"area_code_list\":[\"86\",\"886\",\"862\"]}"
        initBean = InitBean.toBean(test)
        Logger.d(initBean)
        callback.onResult(0, "")
        initCallback.onResult(0, "")
    }

    fun login(activity: Activity, isAutoLogin: Boolean, callback: ICallback) {
        LoginActivity.login(activity, isAutoLogin, object : IImplCallback {
            override fun onResult(code: Int, result: String) {
                Logger.d("onResult code : $code , result : $result")
                ToastUtils.toastInfo(activity, "onResult code : $code , result : $result")
            }
        })
    }

    fun logout(activity: Activity, callback: ICallback) {

    }

    fun charge(activity: Activity, chargeInfo: SdkChargeInfo, callback: ICallback) {

    }

    fun roleCreate(activity: Activity, roleInfo: SdkRoleInfo) {

    }

    fun roleLauncher(activity: Activity, roleInfo: SdkRoleInfo) {

    }

    fun roleLevelUp(activity: Activity, roleInfo: SdkRoleInfo) {

    }

    fun openExitView(activity: Activity, callback: ICallback) {

    }

    fun onStart(activity: Activity) {

    }

    fun onRestart(activity: Activity) {

    }

    fun onResume(activity: Activity) {

    }

    fun onPause(activity: Activity) {

    }

    fun onStop(activity: Activity) {

    }

    fun onDestroy(activity: Activity) {

    }

    fun onActivityResult(activity: Activity, requestCode: Int, resultCode: Int, intent: Intent) {

    }

    fun onNewIntent(activity: Activity, intent: Intent) {

    }

    fun getCurrentUserId(): String {
        return SdkBackLoginInfo.instance.userId
    }

    companion object {

        var isLand = false

        lateinit var initBean: InitBean

    }

}