package cn.yyxx.columbus.core.impl.login

import android.app.Activity
import android.content.Intent
import cn.yyxx.columbus.base.utils.Logger
import cn.yyxx.columbus.base.utils.ParamsUtils
import cn.yyxx.columbus.core.entity.LoginType
import cn.yyxx.support.hawkeye.ToastUtils
import com.facebook.*
import com.facebook.login.LoginManager
import com.facebook.login.LoginResult
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInAccount
import com.google.android.gms.auth.api.signin.GoogleSignInClient
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import com.google.android.gms.common.ConnectionResult
import com.google.android.gms.common.GoogleApiAvailability
import com.google.android.gms.common.api.ApiException
import com.google.android.gms.tasks.Task
import org.json.JSONException
import org.json.JSONObject
import java.util.*

/**
 * @author #Suyghur.
 * Created on 2021/06/18
 */
class UserSignInImpl constructor(val activity: LoginActivity, val callback: ISignInCallback) {

    private var fbCallback: CallbackManager? = null
    private var googleSignInClient: GoogleSignInClient? = null

    init {
        this.fbCallback = CallbackManager.Factory.create()
        val googleServerClientId = ParamsUtils.getGoogleClientId(activity)
        val gso = GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
            .requestEmail()
            .requestIdToken(googleServerClientId)
            .requestId()
            .requestProfile()
            .build()
        this.googleSignInClient = GoogleSignIn.getClient(activity, gso)
    }

    fun facebookLogin(activity: Activity) {
        if (!FacebookSdk.isInitialized()) {
            callback.onFailed("Facebook SDK 初始化失败")
        }
        //先注销账号，防止同一台设备有多个应用登录后直接回调onCancel
        LoginManager.getInstance().logOut()
        LoginManager.getInstance().registerCallback(fbCallback, object : FacebookCallback<LoginResult> {
            override fun onSuccess(result: LoginResult?) {
                result?.apply {
                    try {
                        val jsonObject = JSONObject()
                        jsonObject.put("login_type", LoginType.TYPE_FACEBOOK_LOGIN)
                        jsonObject.put("third_plat_id", AccessToken.getCurrentAccessToken().userId)
                        jsonObject.put("third_plat_token", AccessToken.getCurrentAccessToken().token)
                        callback.onSuccess(jsonObject.toString())
                    } catch (e: JSONException) {
                        e.printStackTrace()
                        callback.onFailed("Facebook登录异常")
                    }
                }
            }

            override fun onCancel() {
                callback.onFailed("Facebook登录取消")
            }

            override fun onError(error: FacebookException?) {
                error?.apply {
                    Logger.e("FBLogin onError error : " + this.message)
                }
                callback.onFailed("Facebook登录失败")
            }
        })
        LoginManager.getInstance().logInWithReadPermissions(activity, Collections.singletonList("public_profile"))
    }

    fun googleLogin(activity: Activity) {
        if (GoogleApiAvailability.getInstance().isGooglePlayServicesAvailable(activity) != ConnectionResult.SUCCESS) {
            ToastUtils.toastInfo(activity, "Google Services is not available for this device")
            callback.onFailed("谷歌登录服务不可用，登录失败")
            return
        }

        googleSignInClient?.apply {
            val intent = signInIntent
            activity.startActivityForResult(intent, 10000)
        }
    }

    private fun handleGoogleSignInResult(data: Intent) {
        val task: Task<GoogleSignInAccount> = GoogleSignIn.getSignedInAccountFromIntent(data)
        try {
            val account = task.getResult(ApiException::class.java)
            Logger.d("${account?.email}")
            try {
                val jsonObject = JSONObject()
                jsonObject.put("login_type", LoginType.TYPE_GOOGLE_LOGIN)
                jsonObject.put("third_plat_id", account?.id)
                jsonObject.put("third_plat_token", account?.idToken)
                callback.onSuccess(jsonObject.toString())
            } catch (e: JSONException) {
                e.printStackTrace()
                callback.onFailed("Google登录异常")
            }
        } catch (e: ApiException) {
            e.printStackTrace()
            callback.onFailed("Google登录异常")
        }
    }


    fun guestLogin(activity: Activity) {

    }

    fun accountLogin(activity: Activity) {

    }

    fun onSignInResult(requestCode: Int, resultCode: Int, intent: Intent) {
        fbCallback?.apply {
            onActivityResult(requestCode, resultCode, intent)
        }
        if (requestCode == 10000) {
            handleGoogleSignInResult(intent)
        }
    }

    interface ISignInCallback {
        fun onSuccess(result: String)

        fun onFailed(result: String)
    }
}